﻿using Excavator.Entities;
using System.Drawing.Drawing2D;
using System.Net.Sockets;

namespace Excavator.Drawnings;

/// <summary>
/// Класс, отвечающий за прорисовку и перемещение объекта-сущности
/// </summary>
public class DrawningExcavator : DrawningTrackedVehicle
{
    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="bucket">Признак наличия ковша</param>
    /// <param name="supports">Признак наличия опор</param>

    public DrawningExcavator(int speed, double weight, Color bodyColor, Color additionalColor, bool bucket, bool supports) : base(150,126)
    {
        EntityTrackedVehicle = new EntityExcavator(speed, weight, bodyColor, additionalColor, bucket, supports);
      
    }

    public override void DrawTransport(Graphics g)
    {
        if (EntityTrackedVehicle == null || EntityTrackedVehicle is not EntityExcavator excavator || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }
        Pen pen = new(Color.Black);
        Brush additionalBrush = new SolidBrush(excavator.AdditionalColor);
        Brush brBlue = new SolidBrush(Color.LightBlue);
        Brush brGray = new SolidBrush(Color.Gray);
        Brush brRed = new SolidBrush(Color.Red);
        Brush brYellow = new SolidBrush(Color.Yellow);
        Brush brBlack = new SolidBrush(Color.Black);



        int bodyHeight = 90;
        int cabineHeight = 40;
        int bucketWidth = 30;
        int wheelsHeight = 40;

        Pen bPen = new(Color.Black);
        bPen.Width = 3;

        _startPosX += 20;
        base.DrawTransport(g);
        _startPosX -= 20;


        // ковш
        if (excavator.Bucket)
        {
            
            PointF p1 = new Point(_startPosX.Value + bucketWidth, _startPosY.Value + cabineHeight);
            PointF p2 = new Point(_startPosX.Value, _startPosY.Value + cabineHeight);
            PointF p3 = new Point(_startPosX.Value + bucketWidth , _startPosY.Value + cabineHeight + bodyHeight / 2);

            g.DrawLine(pen, p1, p2);
            g.DrawLine(pen, p2, p3);
            g.DrawLine(pen, p3, p1);

            g.FillPolygon(additionalBrush, [p1, p2, p3]);
            
        }

        // опоры
        if (excavator.Supports)
        {
            
            g.DrawLine(bPen,
                _startPosX.Value + bucketWidth + bodyHeight, _startPosY.Value + cabineHeight + 10,
                _startPosX.Value + bucketWidth + bodyHeight + 15, _startPosY.Value + cabineHeight + 10);
            g.DrawLine(bPen,
                _startPosX.Value + bucketWidth + bodyHeight +15, _startPosY.Value + cabineHeight + 10,
                _startPosX.Value + bucketWidth + bodyHeight + 15, _startPosY.Value + bodyHeight + wheelsHeight);
            g.DrawLine(bPen,
                _startPosX.Value + bucketWidth + bodyHeight + 15, _startPosY.Value + bodyHeight + wheelsHeight,
                _startPosX.Value + bucketWidth + bodyHeight + 30, _startPosY.Value + bodyHeight + wheelsHeight);
        }

    }   
}

