﻿using Excavator.Drawnings;
using Excavator.MovementStrategy;

namespace Excavator;

/// <summary>
/// Форма работы с объектом "Экскаватор"
/// </summary>
public partial class FormExcavator : Form
{
    /// <summary>
    /// Поле-объект для прорисовки объекта
    /// </summary>
    private DrawningTrackedVehicle? _drawningTrackedVehicle;
    /// <summary>
    ///Стратегия перемещения 
    /// </summary>
    private AbstractStrategy? _strategy;

    public DrawningTrackedVehicle SetTrackedVehicle
    {
        set
        {
            _drawningTrackedVehicle = value;
            _drawningTrackedVehicle.SetPictureSize(pictureBoxExcavator.Width, pictureBoxExcavator.Height);
            ComboBoxStrategy.Enabled = true;
            _strategy = null;
            Draw();
        }
    }

    /// <summary>
    /// Конструктор формы
    /// </summary>
    public FormExcavator()
    {
        InitializeComponent();
        _strategy = null;
    }
    /// <summary>
    /// Метод прорисовки экскаватора
    /// </summary>

    private void Draw()
    {
        if (_drawningTrackedVehicle == null)
        {
            return;
        }
        Bitmap bmp = new(pictureBoxExcavator.Width, pictureBoxExcavator.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _drawningTrackedVehicle.DrawTransport(gr);
        pictureBoxExcavator.Image = bmp;

    }

    /// <summary>
	/// Создание объекта класса-перемещения
	/// </summary>
	/// <param name="type">Тип создаваемого объекта</param>
	private void CreateObject(string type)
    {
        Random random = new();
        switch (type)
        {
            case nameof(DrawningTrackedVehicle):
                _drawningTrackedVehicle = new DrawningTrackedVehicle(random.Next(100, 300), random.Next(1000, 3000),
                    Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)));
                break;
            case nameof(DrawningExcavator):
                _drawningTrackedVehicle = new DrawningExcavator(random.Next(100, 300), random.Next(1000, 3000),
                    Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
                    Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
                    Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)));
                break;
            default:
                return;
        }

        _drawningTrackedVehicle.SetPictureSize(pictureBoxExcavator.Width, pictureBoxExcavator.Height);
        _drawningTrackedVehicle.SetPosition(random.Next(0, 100), random.Next(0, 100));
        _strategy = null;
        ComboBoxStrategy.Enabled = true;
        Draw();
    }



    /// <summary>
    /// Обработка нажатия кнопки "Создать экскаватор"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonCreateExcavator_Click(object sender, EventArgs e) => CreateObject(nameof(DrawningExcavator));




    /// <summary>
    /// Обработка нажатия кнопки "Создать гусеничную машину"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonCreateTrackedVehicle_Click(object sender, EventArgs e) => CreateObject(nameof(DrawningTrackedVehicle));

    /// <summary>
    /// Перемещение объекта по форме (нажатие кнопок навигации)
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonMove_Click(object sender, EventArgs e)
    {
        if (_drawningTrackedVehicle == null)
        {
            return;
        }

        string name = ((Button)sender)?.Name ?? string.Empty;
        bool result = false;
        switch (name)
        {
            case "buttonUp":
                result = _drawningTrackedVehicle.MoveTransport(DirectionType.Up);
                break;
            case "buttonDown":
                result = _drawningTrackedVehicle.MoveTransport(DirectionType.Down);
                break;
            case "buttonLeft":
                result = _drawningTrackedVehicle.MoveTransport(DirectionType.Left);
                break;
            case "buttonRight":
                result = _drawningTrackedVehicle.MoveTransport(DirectionType.Right);
                break;
        }

        if (result)
        {
            Draw();
        }
    }




    /// <summary>
    /// 
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonStrategyStep_Click(object sender, EventArgs e)
    {
        if (_drawningTrackedVehicle == null)
        {
            return;
        }

        if (ComboBoxStrategy.Enabled)
        {
            _strategy = ComboBoxStrategy.SelectedIndex switch
            {
                0 => new MoveToCenter(),
                1 => new MoveToBorder(),
                _ => null,
            };
            if (_strategy == null)
            {
                return;
            }
            _strategy.SetData(new MoveableTrackedVehicle(_drawningTrackedVehicle), pictureBoxExcavator.Width, pictureBoxExcavator.Height);
        }

        if (_strategy == null)
        {
            return;
        }

        ComboBoxStrategy.Enabled = false;
        _strategy.MakeStep();
        Draw();

        if (_strategy.GetStatus() == StrategyStatus.Finish)
        {
            ComboBoxStrategy.Enabled = true;
            _strategy = null;
        }
    }
}


