﻿using Excavator.Drawnings;
using Excavator.Entities;

namespace Excavator;

/// <summary>
/// Форма конфигурации объекта
/// </summary>
public partial class FormTrackedVehicleConfig : Form
/// <summary>
/// Объект - прорисовка гусеничной машины
/// </summary>

{
    private DrawningTrackedVehicle? _car;
    private event Action<DrawningTrackedVehicle>? CarDelegate;

    /// <summary>
    /// Событие для передачи объекта
    /// </summary>
    /// <param name="trainDelegate"></param>
    public void AddEvent(Action<DrawningTrackedVehicle> carDelegate)
    {
        if (CarDelegate != null)
        {
            CarDelegate = carDelegate;
        }
        else
        {
            CarDelegate += carDelegate;
        }
    }

    /// <summary>
    /// Конструктор
    /// </summary>
    public FormTrackedVehicleConfig()
    {
        InitializeComponent();
        panelRed.MouseDown += Panel_MouseDown;
        panelGreen.MouseDown += Panel_MouseDown;
        panelBlue.MouseDown += Panel_MouseDown;
        panelYellow.MouseDown += Panel_MouseDown;
        panelWhite.MouseDown += Panel_MouseDown;
        panelGray.MouseDown += Panel_MouseDown;
        panelBlack.MouseDown += Panel_MouseDown;
        panelPurple.MouseDown += Panel_MouseDown;

        buttonCancel.Click += (sender, e) => Close();

    }

    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    private void DrawObject()
    {
        Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _car?.SetPictureSize(pictureBoxObject.Width, pictureBoxObject.Height);
        _car?.SetPosition(15, 15);
        _car?.DrawTransport(gr);
        pictureBoxObject.Image = bmp;
    }
    /// <summary>
	/// Передаем информацию при нажатии на Label
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
    private void labelObject_MouseDown(object sender, MouseEventArgs e)
    {
        (sender as Label)?.DoDragDrop((sender as Label)?.Name ?? string.Empty, DragDropEffects.Move | DragDropEffects.Copy);
    }
    /// <summary>
	/// Проверка получаемой информации (ее типа на соответствие требуемому)
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
    private void panelObject_DragEnter(object sender, DragEventArgs e)
    {
        e.Effect = e.Data?.GetDataPresent(DataFormats.Text) ?? false ? DragDropEffects.Copy : DragDropEffects.None;
    }
    /// <summary>
	/// Действия при приеме перетаскиваемой информации
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
    private void panelObject_DragDrop(object sender, DragEventArgs e)
    {
        switch (e.Data?.GetData(DataFormats.Text)?.ToString())
        {
            case "labelSimpleObject":
                _car = new DrawningTrackedVehicle((int)numericUpDownSpeed.Value, (double)numericUpDownWeight.Value, Color.White);
                break;
            case "labelModifiedObject":
                _car = new DrawningExcavator((int)numericUpDownSpeed.Value, (double)numericUpDownWeight.Value, Color.White,
                    Color.Black, checkBoxBucket.Checked, checkBoxSupports.Checked);
                break;
        }

        DrawObject();
    }

    /// <summary>
    /// Передаем информацию при нажатии на Panel
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void Panel_MouseDown(object? sender, MouseEventArgs e)
    {
        (sender as Control)?.DoDragDrop((sender as Control).BackColor, DragDropEffects.Move | DragDropEffects.Copy);
    }
    private void labelBodyColor_DragEnter(object? sender, DragEventArgs e)
    {
        if (e.Data.GetDataPresent(typeof(Color)))
        {
            e.Effect = DragDropEffects.Copy;
        }
        else
        {
            e.Effect = DragDropEffects.None;
        }
    }

    private void labelBodyColor_DragDrop(object? sender, DragEventArgs e)
    {
        if (_car != null)
        {
            _car.EntityTrackedVehicle.SetBodyColor((Color)e.Data.GetData(typeof(Color)));
            DrawObject();
        }
    }
    private void labelAdditionalColor_DragEnter(object? sender, DragEventArgs e)
    {
        if (_car is DrawningExcavator)
        {
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
    }

    private void labelAdditionalColor_DragDrop(object? sender, DragEventArgs e)
    {
        if (_car.EntityTrackedVehicle is EntityExcavator excavator)
        {
            excavator.SetAdditionalColor((Color)e.Data.GetData(typeof(Color)));
            DrawObject();
        }
    }
    /// <summary>
    /// Передача объекта
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonAdd_Click(object sender, EventArgs e)
    {
        if (_car != null)
        {
            CarDelegate?.Invoke(_car);
            Close();
        }
    }

}
