﻿using Excavator.Drawnings;


namespace Excavator.MovementStrategy;
/// <summary>
/// Класс-реализация IMoveableObject с использованием DrawningTrackedVehicle
/// </summary>
public class MoveableTrackedVehicle : IMoveableObject

{
    /// <summary>
	/// Поле-объект класса DrawningTrackedVehicle или его наследника
	/// </summary>
	private readonly DrawningTrackedVehicle? _trackedvehicle = null;

    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="car">Объект класса DrawningCar</param>
    public MoveableTrackedVehicle(DrawningTrackedVehicle trackedvehicle)
    {
        _trackedvehicle = trackedvehicle;
    }

    public ObjectParameters? GetObjectPosition
    {
        get
        {
            if (_trackedvehicle == null || _trackedvehicle.EntityTrackedVehicle == null || !_trackedvehicle.GetPosX.HasValue || !_trackedvehicle.GetPosY.HasValue)
            {
                return null;
            }
            return new ObjectParameters(_trackedvehicle.GetPosX.Value, _trackedvehicle.GetPosY.Value, _trackedvehicle.GetWidth, _trackedvehicle.GetHeight);
        }
    }

    public int GetStep => (int)(_trackedvehicle?.EntityTrackedVehicle?.Step ?? 0);

    public bool TryMoveObject(MovementDirection direction)
    {
        if (_trackedvehicle == null || _trackedvehicle.EntityTrackedVehicle == null)
        {
            return false;
        }

        return _trackedvehicle.MoveTransport(GetDirectionType(direction));
    }

    /// <summary>
    /// Конвертация из MovementDirection в DirectionType
    /// </summary>
    /// <param name="direction">MovementDirection</param>
    /// <returns>DirectionType</returns>
    private static DirectionType GetDirectionType(MovementDirection direction)
    {
        return direction switch
        {
            MovementDirection.Left => DirectionType.Left,
            MovementDirection.Right => DirectionType.Right,
            MovementDirection.Up => DirectionType.Up,
            MovementDirection.Down => DirectionType.Down,
            _ => DirectionType.Unknow,
        };
    }
}
